@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# Copyright (c) 2011, 2012, 2013.
+# Copyright (c) 2011, 2012, 2013, 2014.
 
 # Author(s):
 
@@ -85,20 +85,6 @@ class Orbital(object):
         self.orbit_elements = OrbitElements(self.tle)
         self._sgdp4 = _SGDP4(self.orbit_elements)
         
-        pos_epoch, vel_epoch = self.get_position(self.tle.epoch, 
-                                                 normalize=False)
-        if np.abs(pos_epoch[2]) > 1 or not vel_epoch[2] > 0:
-            # Epoch not at ascending node
-            self.orbit_elements.an_time = self.get_last_an_time(self.tle.epoch)
-        else:
-            # Epoch at ascending node (z < 1 km) and positive v_z
-            self.orbit_elements.an_time = self.tle.epoch
-            
-        self.orbit_elements.an_period = self.orbit_elements.an_time - \
-                        self.get_last_an_time(self.orbit_elements.an_time 
-                                              - timedelta(minutes=10))    
-            
-
     def __str__(self):
         return self.satellite_name + " " + str(self.tle)
 
@@ -177,38 +163,35 @@ class Orbital(object):
         alt *= A
         return np.rad2deg(lon), np.rad2deg(lat), alt
 
-    def find_aos(self, time, lon, lat):
+    def find_aos(self, utc_time, lon, lat):
         pass
 
-    def find_aol(self, time, lon, lat):
+    def find_aol(self, utc_time, lon, lat):
         pass
 
-    def get_observer_look(self, time, lon, lat, alt):
+    def get_observer_look(self, utc_time, lon, lat, alt):
         """Calculate observers look angle to a satellite.
         http://celestrak.com/columns/v02n02/
 
-        time: Observation time (datetime object)
+        utc_time: Observation time (datetime object)
         lon: Longitude of observer position on ground
         lat: Latitude of observer position on ground
         alt: Altitude above sea-level (geoid) of observer position on ground
 
         Return: (Azimuth, Elevation)
         """
-        (pos_x, pos_y, pos_z), (vel_x, vel_y, vel_z) = self.get_position(time, normalize=False)
+        (pos_x, pos_y, pos_z), (vel_x, vel_y, vel_z) = self.get_position(utc_time, normalize=False)
         (opos_x, opos_y, opos_z), (ovel_x, ovel_y, ovel_z) = \
-                                    astronomy.observer_position(time, lon, lat, alt)
+                                    astronomy.observer_position(utc_time, lon, lat, alt)
         
         lon = np.deg2rad(lon)
         lat = np.deg2rad(lat)
         
-        theta = (astronomy.gmst(time) + lon) % (2 * np.pi)
+        theta = (astronomy.gmst(utc_time) + lon) % (2 * np.pi)
 
         rx = pos_x - opos_x
         ry = pos_y - opos_y
         rz = pos_z - opos_z
-        rvx = vel_x - ovel_x
-        rvy = vel_y - ovel_y
-        rvz = vel_z - ovel_z
 
         sin_lat = np.sin(lat)
         cos_lat = np.cos(lat)
@@ -219,25 +202,40 @@ class Orbital(object):
         top_e = -sin_theta * rx + cos_theta * ry
         top_z = cos_lat * cos_theta * rx + cos_lat * sin_theta * ry + sin_lat * rz
 
-        az = np.arctan(-top_e / top_s)
-        if top_s > 0:
-            az = az + np.pi
-        if az < 0:
-            az = az + 2 * np.pi
+        az_ = np.arctan(-top_e / top_s)
 
-        rg = np.sqrt(rx * rx + ry * ry + rz * rz)
-        el = np.arcsin(top_z / rg)
-        w = (rx * rvx + ry * rvy + rz * rvz) / rg
+        az_ = np.where(top_s > 0, az_ + np.pi, az_)
+        az_ = np.where(az_ < 0, az_ + 2 * np.pi, az_)
 
-        return np.rad2deg(az), np.rad2deg(el)
+        rg_ = np.sqrt(rx * rx + ry * ry + rz * rz)
+        el_ = np.arcsin(top_z / rg_)
+
+        return np.rad2deg(az_), np.rad2deg(el_)
 
     def get_orbit_number(self, utc_time, tbus_style=False):
         """Calculate orbit number at specified time.
         Optionally use TBUS-style orbit numbering (TLE orbit number + 1)
         """
+        try:        
+            dt = astronomy._days(utc_time - self.orbit_elements.an_time)
+            orbit_period = astronomy._days(self.orbit_elements.an_period)
+        except AttributeError:
+            pos_epoch, vel_epoch = self.get_position(self.tle.epoch, 
+                                                 normalize=False)
+            if np.abs(pos_epoch[2]) > 1 or not vel_epoch[2] > 0:
+                # Epoch not at ascending node
+                self.orbit_elements.an_time = self.get_last_an_time(self.tle.epoch)
+            else:
+                # Epoch at ascending node (z < 1 km) and positive v_z
+                self.orbit_elements.an_time = self.tle.epoch
+            
+            self.orbit_elements.an_period = self.orbit_elements.an_time - \
+                                            self.get_last_an_time(self.orbit_elements.an_time 
+                                                                  - timedelta(minutes=10))    
+
+            dt = astronomy._days(utc_time - self.orbit_elements.an_time)
+            orbit_period = astronomy._days(self.orbit_elements.an_period)
         
-        dt = astronomy._days(utc_time - self.orbit_elements.an_time)
-        orbit_period = astronomy._days(self.orbit_elements.an_period)
                  
         orbit = int(self.tle.orbit + dt / orbit_period + 
                  self.tle.mean_motion_derivative * dt**2 + 
@@ -246,97 +244,67 @@ class Orbital(object):
         if tbus_style:
             orbit += 1
         return orbit
-        
-    def get_zenith_overpass(self, utc_time, obslon, obslat):
-        """Get the time when the satellite is highest on the horizon relative
-        to the observer position on ground given by *obslon*,*obslat* closest
-        in time (in the future) to the time given by *utc_time*. Using the
-        method of gradient ascent with variable step parameter (decreasing in
-        size as we apprach 90 degrees zenith angle.
+
+    def get_next_passes(self, utc_time, length, lon, lat, alt, tol=0.001):
+        """Calculate passes for the next hours for a given start time and a 
+        given observer.
+
+        Original by Martin.
+
+        utc_time: Observation time (datetime object)
+        length: Number of hours to find passes (int)
+        lon: Longitude of observer position on ground (float)
+        lat: Latitude of observer position on ground (float)
+        alt: Altitude above sea-level (geoid) of observer position on ground (float)
+        tol: precision of the result in seconds
+
+        Return: [(rise-time, fall-time, max-elevation-time), ...]
         """
+        from scipy.optimize import brentq, brent
 
-        # First check if the elevation is above zero. If not continue until it
-        # is, using larger steps when the ange is far from zero and shorter
-        # ones when the elevation gets closer to zero:
-        el_start = self.get_observer_look(utc_time, obslon, obslat, 0.0)[1]
-        if el_start < 0:
-            start_time = utc_time
-            elev = el_start
-            idx = 0
-            NIDX = 100
-            while elev < 0 and idx < NIDX:
-                var_scale = np.exp(np.square(np.sin(elev * np.pi/180.)))
-                t_step = timedelta(seconds = (300 * var_scale))
-                start_time = start_time + t_step
-                elev = self.get_observer_look(start_time, obslon, obslat, 0.0)[1]
-                idx = idx + 1
-                #print idx, start_time, var_scale, elev
+        def elevation(minutes):
+            return self.get_observer_look(utc_time +
+                                          timedelta(minutes=minutes),
+                                          lon, lat, alt)[1]
+        def elevation_inv(minutes):
+            return -elevation(minutes)
 
-            utc_time = start_time - t_step
+        times = utc_time + np.array([timedelta(minutes=minutes)
+                                    for minutes in range(length * 60)])
+        elev = self.get_observer_look(times, lon, lat, alt)[1]
+        a = np.where(np.diff(np.sign(elev)))[0]
 
-        precision = timedelta(seconds=0.01)
-        sec_step = 0.5
-        t_step = timedelta(seconds=sec_step/2.0)
+        res = []
+        risetime = None
+        falltime = None
+        for guess in a:
+            horizon_mins = brentq(elevation, guess, guess + 1, xtol=tol/60.0)
+            horizon_time = utc_time + timedelta(minutes=horizon_mins)
+            if elev[guess] < 0:
+                risetime = horizon_time
+                risemins = horizon_mins
+                falltime = None
+            else:
+                falltime = horizon_time
+                fallmins = horizon_mins
+                if risetime:
+                    middle = (risemins + fallmins) / 2.0
+                    highest = utc_time + \
+                        timedelta(minutes=brent(
+                            elevation_inv, 
+                            brack=(middle - 0.1, middle + 0.1),
+                            tol=tol/60.0))
+                    res += [(risetime, falltime, highest)]
+                risetime = None
+        return res
 
-        # Local derivative:
-        def fprime(timex):
-            el0 = self.get_observer_look(timex - t_step, 
-                                         obslon, obslat, 0.0)[1]
-            el1 = self.get_observer_look(timex + t_step, 
-                                         obslon, obslat, 0.0)[1]
-            return el0, (el1 - el0) / sec_step 
-
-        tx0 = utc_time - timedelta(seconds=1.0)
-        tx1 = utc_time
-        idx = 0
-        NIDX = 100
-        eps = 1000. # Step size scaling
-        while abs(tx1 - tx0) > precision and idx < NIDX:
-            tx0 = tx1
-            fpr = fprime(tx0)
-            #var_scale = abs(90.0-fpr[0])
-            var_scale = np.abs(np.cos(fpr[0] * np.pi/180.))
-            tx1 = tx0 + timedelta(seconds = (eps * var_scale * fpr[1]))
-            #print idx, tx0, tx1, fpr
-            idx = idx + 1
-    
-        if abs(tx1 - tx0) <= precision and idx < NIDX:
-            return tx1, idx
-        else:
-            return None
-        
-    def get_risetime(self, utc_time, obslon, obslat, **kwargs):
-        """Get the risetime of the satellite closest in time in the future to
-        the time *utc_time* for a reception station at *obslon*, *obslat*
-        position on the ground
-        """
-        retv = self.get_zenith_overpass(utc_time, obslon, obslat)
-        if retv:
-            zenith_time = retv[0]
-        else:
-            zenith_time = utc_time
-        one_minute = timedelta(seconds = 60)
-        return self._get_time_at_horizon(zenith_time - one_minute, 
-                                         obslon, obslat, **kwargs)
-        
-    def get_falltime(self, utc_time, obslon, obslat, **kwargs):
-        """Get the falltime of the satellite closest in time in the future to
-        the time *utc_time* for a reception station at *obslon*, *obslat*
-        position on the ground
-        """
-        retv = self.get_zenith_overpass(utc_time, obslon, obslat)
-        if retv:
-            zenith_time = retv[0]
-        else:
-            zenith_time = utc_time
-        one_minute = timedelta(seconds = 60)
-        return self._get_time_at_horizon(zenith_time + one_minute, 
-                                         obslon, obslat, **kwargs)
-        
     def _get_time_at_horizon(self, utc_time, obslon, obslat, **kwargs):
         """Get the time closest in time to *utc_time* when the
         satellite is at the horizon relative to the position of an observer on
         ground (altitude = 0)
+
+        Note: This is considered deprecated and it's functionality is currently
+        replaced by 'get_next_passes'.
         """
         if "precision" in kwargs:
             precision = kwargs['precision']
@@ -382,7 +350,6 @@ class Orbital(object):
         else:
             return None
 
-
 class OrbitElements(object):
     """Class holding the orbital elements.
     """
